/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (C) 2024, STMicroelectronics - All Rights Reserved
 */

#ifndef __DT_BINDINGS_REGULATOR_ST_STM32MP25_REGULATOR_H
#define __DT_BINDINGS_REGULATOR_ST_STM32MP25_REGULATOR_H

/* SCMI voltage domains identifiers */

/* SOC Internal regulators */
#define VOLTD_SCMI_VDDIO1		0
#define VOLTD_SCMI_VDDIO2		1
#define VOLTD_SCMI_VDDIO3		2
#define VOLTD_SCMI_VDDIO4		3
#define VOLTD_SCMI_VDDIO		4
#define VOLTD_SCMI_UCPD			5
#define VOLTD_SCMI_USB33		6
#define VOLTD_SCMI_ADC			7
#define VOLTD_SCMI_GPU			8
#define VOLTD_SCMI_VREFBUF		9

/* STPMIC2 regulators */
#define VOLTD_SCMI_STPMIC2_BUCK1	10
#define VOLTD_SCMI_STPMIC2_BUCK2	11
#define VOLTD_SCMI_STPMIC2_BUCK3	12
#define VOLTD_SCMI_STPMIC2_BUCK4	13
#define VOLTD_SCMI_STPMIC2_BUCK5	14
#define VOLTD_SCMI_STPMIC2_BUCK6	15
#define VOLTD_SCMI_STPMIC2_BUCK7	16
#define VOLTD_SCMI_STPMIC2_LDO1		17
#define VOLTD_SCMI_STPMIC2_LDO2		18
#define VOLTD_SCMI_STPMIC2_LDO3		19
#define VOLTD_SCMI_STPMIC2_LDO4		20
#define VOLTD_SCMI_STPMIC2_LDO5		21
#define VOLTD_SCMI_STPMIC2_LDO6		22
#define VOLTD_SCMI_STPMIC2_LDO7		23
#define VOLTD_SCMI_STPMIC2_LDO8		24
#define VOLTD_SCMI_STPMIC2_REFDDR	25

/* External regulators */
#define VOLTD_SCMI_REGU0		26
#define VOLTD_SCMI_REGU1		27
#define VOLTD_SCMI_REGU2		28
#define VOLTD_SCMI_REGU3		29
#define VOLTD_SCMI_REGU4		30

#endif /*__DT_BINDINGS_REGULATOR_ST_STM32MP25_REGULATOR_H */
