/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2021, The Linux Foundation. All rights reserved.
 */

#ifndef _DT_BINDINGS_CLK_QCOM_LPASS_SC7280_H
#define _DT_BINDINGS_CLK_QCOM_LPASS_SC7280_H

#define LPASS_Q6SS_AHBM_CLK				0
#define LPASS_Q6SS_AHBS_CLK				1
#define LPASS_TOP_CC_LPI_Q6_AXIM_HS_CLK			2
#define LPASS_QDSP6SS_XO_CLK				3
#define LPASS_QDSP6SS_SLEEP_CLK				4
#define LPASS_QDSP6SS_CORE_CLK				5

#endif
