/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_BINDINGS_POWER_RK3128_POWER_H__
#define __DT_BINDINGS_POWER_RK3128_POWER_H__

/* VD_CORE */
#define RK3128_PD_CORE		0

/* VD_LOGIC */
#define RK3128_PD_VIO		1
#define RK3128_PD_VIDEO		2
#define RK3128_PD_GPU		3
#define RK3128_PD_MSCH		4

#endif
